import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
import { createOpenAiClient } from './app';
import { OpenAi, PlatformEventEmitter } from '@modal-sh/mio-ai';

export type Argv = Record<string, unknown>;

const receiveData = (aiClient: PlatformEventEmitter, theContent: string, argv: Record<string, unknown>) => new Promise<void>((r2, rj) => {
  let countTokens: number;
  aiClient.on<OpenAi.ChatCompletionChunkDataEvent>('data', (d) => {
    if (d.choices?.[0]?.delta?.role) {
      process.stdout.write(`${d.choices[0].delta.role}: `);
    }

    if (d.choices?.[0]?.delta?.content) {
      countTokens += 1;
      process.stdout.write(d.choices[0].delta.content);
    }
  });

  aiClient.on('end', () => {
    process.stdout.write('\n\n');
    process.stdout.write(`info: ${countTokens} completion tokens.\n\n`);
    // TODO count tokens
    r2();
  });

  aiClient.on('error', (error: Error) => {
    process.stderr.write(error.message);
    process.stderr.write('\n');
    rj(error);
  });

  countTokens = 0;
  aiClient.createChatCompletion({
    model: OpenAi.ChatCompletionModel.GPT_3_5_TURBO,
    messages: theContent,
    temperature: argv.temperature as number,
    maxTokens: argv.maxTokens as number,
    topP: argv.topP as number,
  });
});

const main = (argv: Argv) => new Promise<number>(async (resolve) => {
  let done = false;
  let resolveResult = 0;

  process.stdout.write('Welcome to mio-ai CLI!\n\n');

  process.stdout.write('This is a simple example of how to use mio-ai.\n');
  process.stdout.write('You can send messages to the OpenAI API.\n');
  process.stdout.write('You can also send empty messages to exit.\n');
  process.stdout.write('Alternatively, you can press Ctrl+C anytime to exit.\n\n');
  do {
    process.stdout.write('---\n\n');
    const aiClient = createOpenAiClient({
      apiKey: process.env.OPENAI_API_KEY as string,
      organizationId: process.env.OPENAI_ORGANIZATION_ID as string,
      apiVersion: OpenAi.ApiVersion.V1,
    });
    const { createPromptModule } = await import('inquirer');
    const prompt = createPromptModule();
    let content: string;
    try {
      const answers = await prompt([
        {
          type: 'input',
          name: 'content',
          prefix: '',
          message: `${OpenAi.MessageRole.USER}:`,
        },
      ]);
      content = answers.content;
      if (content.trim().length < 1) {
        done = true;
        break;
      }
      process.stdout.write('\n');
      await receiveData(aiClient, content, argv);
    } catch {
      resolveResult = -1;
      done = true;
    }
  } while (!done);

  process.stdout.write('Bye!\n');
  resolve(resolveResult);
});

main(yargs(hideBin(process.argv)).argv as unknown as Argv)
  .then((result) => {
    process.exit(result);
  })
  .catch((error) => {
    console.error(error);
    process.exit(-1);
  });
