import { PassThrough } from 'stream';
import { EventEmitter } from 'events';
import fetchPonyfill from 'fetch-ponyfill';
import { Configuration } from './common';
import { createTextCompletion, CreateTextCompletionParams } from './features/text-completion';
import { CreateChatCompletionParams, createChatCompletion } from './features/chat-completion';
import { CreateImageParams, createImage } from './features/image';
import { CreateEditParams, createEdit } from './features/edit';

export type DataEventCallback<D> = (data: D) => void;

export type ErrorEventCallback = (event: Error) => void;

export interface PlatformEventEmitter extends NodeJS.EventEmitter {
  createChatCompletion(params: CreateChatCompletionParams): void;
  createImage(params: CreateImageParams): void;
  createCompletion(params: CreateTextCompletionParams): void;
  createEdit(params: CreateEditParams): void;
  on<D>(event: 'data', callback: DataEventCallback<D>): this;
  on(event: 'end', callback: () => void): this;
  on(event: 'error', callback: ErrorEventCallback): this;
}

export class PlatformEventEmitterImpl extends EventEmitter implements PlatformEventEmitter {
  readonly createCompletion: PlatformEventEmitter['createCompletion'];

  readonly createImage: PlatformEventEmitter['createImage'];

  readonly createChatCompletion: PlatformEventEmitter['createChatCompletion'];

  readonly createEdit: PlatformEventEmitter['createEdit'];

  constructor(configParams: Configuration) {
    super();
    const headers: Record<string, string> = {
      Authorization: `Bearer ${configParams.apiKey}`,
    };

    if (configParams.organizationId) {
      headers['OpenAI-Organization'] = configParams.organizationId;
    }

    const { fetch: fetchInstance } = fetchPonyfill();
    const doFetch = (method: string, path: string, body: Record<string, unknown>) => {
      const theFetchParams = {
        method,
        headers: {
          ...headers,
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(body),
      };

      const url = new URL(
        `/${configParams.apiVersion}${path}`,
        configParams.baseUrl ?? 'https://api.openai.com',
      ).toString();

      this.emit('start', {
        ...theFetchParams,
        url,
      });

      return fetchInstance(url, theFetchParams);
    };

    const consumeStream = async (response: Response) => {
      // eslint-disable-next-line no-restricted-syntax
      for await (const chunk of response.body as unknown as PassThrough) {
        const chunkStringMaybeMultiple = chunk.toString();
        const chunkStrings = chunkStringMaybeMultiple
          .trim()
          .split('\n')
          .filter((chunkString: string) => chunkString.length > 0);
        chunkStrings.forEach((chunkString: string) => {
          const dataRaw = chunkString.split('data: ').at(1);
          if (!dataRaw) {
            return;
          }
          if (dataRaw === '[DONE]') {
            return;
          }
          const data = JSON.parse(dataRaw);
          this.emit('data', data);
        });
      }
    };
    this.createImage = createImage.bind(this, doFetch);
    this.createCompletion = createTextCompletion.bind(this, doFetch, consumeStream);
    this.createChatCompletion = createChatCompletion.bind(this, doFetch, consumeStream);
    this.createEdit = createEdit.bind(this, doFetch);
  }
}
